﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class TurnRight : CodeNode {
    public override int MinimumArity { get { return 1; } }
    public override int MaximumArity { get { return 1; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    private TurnRight(bool deserializing) : base(deserializing) { }
    private TurnRight(TurnRight original, Cloner cloner)
      : base(original, cloner) {

    }

    public TurnRight()
      : base("TurnRight", "Immediately turns the robot's body to the right by degrees.") {
      this.Prefix = "setTurnRight(";
      this.Suffix = ");";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TurnRight(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children) {
      var enumerator = children.GetEnumerator();

      if (!enumerator.MoveNext()) throw new System.Exception("TurnRight was not given a child.");
      var symbol = enumerator.Current.Symbol;

      if (!(symbol is Number || symbol is NumericalOperation || symbol is NumericalExpression))
        throw new System.Exception("TurnRight was given a child of type " + symbol.GetType().ToString() +
            ". The expected child must be of type " + typeof(Number).ToString() + " or " +
            typeof(NumericalOperation).ToString() + " or " +
            typeof(NumericalExpression).ToString() + ".");

      string result = ((CodeNode)symbol).Interpret(enumerator.Current, enumerator.Current.Subtrees);
      if (enumerator.MoveNext()) throw new System.Exception("TurnRight was given more than one child.");

      return this.Prefix + result + this.Suffix;
    }
  }
}